/***************************************************************************
                          directory.cpp  -  description
                             -------------------
    begin                : Sat Jul 1 2000
    copyright            : (C) 2000 by Calvin Harrigan
    email                : charrig@viaccess.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// See header file for details.

#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <iostream.h>
#include "directory.h"

#define random(num)(int) ((long)rand()%(num))
#define randomize() srand((unsigned)time(NULL))


Directory::Directory(char* directory)
{
	// class variables.
	fFlag = false;          // flags failure of constructor
	songListSize = 0;       // keeps count of songs
	list = NULL;            // pointer to list of Song structures
	available = true;       // flags a directory object as being available for use
  // local variables
	DIR * dp;               // directory pointer
	struct dirent *ep;      // directory entry pointer
	int n = 0;              // used to count entries in the directory.
	int i;                  // general purpose index counter.
	int songNameLength = 0; // length of currently indexed song filename
	char * subString;       // used for removing mp3 extensions
	char * fExtension = "3";

	strcpy(name,directory); // saves the name of the directory to a class variable
	
	dp = opendir(directory);// opens the directory and checks for success
	if(dp != NULL)          // if opened it counts all the files that end
	{                       // with '3' usually mp3 files
		while((ep = readdir(dp)))
		{                     // here we actually do the string comparison
			if(!strncmp(fExtension,&ep->d_name[strlen(ep->d_name)-1],1))
				n++;              // a counter is incremented everytime we find one
		}
	}
	else     // if opening the directory fails we set the fail flag and return
	{
		fFlag = true;
		return;
	}

	
	if(n > 0)    // if songs were found we create an array of the amount found
	{
		list =  new Song* [n];  //memory allocation is checked for success.
		if(list == NULL)
		{
			fFlag = true;
			return;
		}
		rewinddir(dp);  // we start at the top of the directory again looking for mp3s
		i = 0;

		while( (ep = readdir(dp)) )
		{
			songNameLength = strlen(ep->d_name);
			list[i] = new Song;   // Song structure space is allocated
			if(list[i] == NULL)
			{	
				fFlag = true;
			}                     // space for the strings are also allocated
			list[i]->name = new char [songNameLength+1];
			list[i]->path = new char [songNameLength + strlen(name)+ 10];
				
			if((list[i]->name == NULL) || (list[i]->path == NULL))
			{                    // if any of the previous 2 allocations failed
				fFlag = true;      // we set the fail flag and return
				return;
			}
			// both strings are cleared to nulls
			memset(list[i]->name,0,songNameLength+1);
			memset(list[i]->path,0,songNameLength + strlen(name)+ 10);

      // here we are looking for files ending in 3s again
			if(!strncmp(fExtension,&ep->d_name[songNameLength-1],1))
			{
			// if found the name is copied to the Song structure along
			// with it's full path, the count of songs also incremented
				strncpy(list[i]->name,ep->d_name,songNameLength);
				// clips the mp3 extension
				subString = strrchr(list[i]->name,'.');
				if(subString != NULL)
					subString[0] = char(NULL);
				// creates the full path
				strcat(list[i]->path,name);
				strcat(list[i]->path,"/");
				strcat(list[i]->path,ep->d_name);
				songListSize++;     // song count
				i++;                // index, usually the same as songList Size
			}
			else
			{ // if not an mp3 file delete the allocated space...
				delete list[i]->name;
				delete list[i]->path;
				delete list[i];
			}
		}			
	}
};

Directory::~Directory()
{
	// Deletes all the allocated space
	// first the song name and paths
	// then the holding structure, Songs
	for(int i = 0; i < songListSize; i++)
	{
		delete list[i]->name;
		delete list[i]->path;
	}
	for(int i = 0; i < songListSize; i++)
	{
		delete list[i];
	}
	delete list;
	list = NULL; // Make the whole list null
}

Song ** Directory::GetSongs()const
{
	// returns the list of Song structures if they exist
	if(list != NULL)
	{
		return list;
	}
	else
	{
		return NULL;
	}
	return NULL;
};

void Directory::GetDirectoryName(char *buffer, int length)const
{
	// copies the name of the directory into a buffer
	strncpy(buffer,name,length);
	return;
};	

