/***************************************************************************
                          display.cpp  -  description
                             -------------------
    begin                : Mon Jun 5 2000
    copyright            : (C) 2000 by 
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// Display class inmplementation.  See display.h for more information.

#include <config.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <stdarg.h>
#include <ctype.h>
#include <time.h>
#include <asm/io.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <termios.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <festival/festival.h>
#include "display.h"
#include "songlist.h"

Display::Display(const char * port)
{
	
	fFlag = false;
	
	serPort = new linSerial();
	if(serPort == NULL)
	{
		fFlag = true;
		return;
	}
	
	if(serPort->OpenDevice(port))
	{
		fFlag = true;
		return;
	}
	festival_initialize(1,210000);
	serPort->Set8N1();
	serPort->SetHardwareFlowControl();
	serPort->SetSpeed(12000);
	serPort->SetDTR();
	serPort->SetupRaw();
	fPort = serPort->GetFileHandle();
	sfd = serPort->GetFileDescriptor();
	contrast = 176;
	contStep = 8;
	quit = false;
	playerReady = false;
	playerState = XA_PLAYER_STATE_STOPPED;
	streamProgress = 0;
	streamMinutes = streamSeconds = 0;
	dispMode = PLAY;
	newXAudioData = false;
	streamInputState = XA_IO_STATE_CLOSED;
	memset(playerSong,0,sizeof(playerSong));
	for(int i = 0; i < 6; i++)
	{
		eqbands.eqbands[i] = 0;
		eqInfo.left[i] = eqInfo.right[i] = 0;
	}
	eqInfo.left[6]= eqInfo.right[6] = eqInfo.left[7] =
		eqInfo.right[7] = eqbands.eqbands[6] = 16;
	eqbands.eqbands[7] = 48;
	for(int i=8;i<=11;i++)
	{
		eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[7];
	}		
	eqbands.eqbands[8] = 96;
	for(int i=12;i<=19;i++)
	{
		eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[8];
	}		
	eqbands.eqbands[9] = 127;
	for(int i=20;i<=31;i++)
	{
		eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[9];
	}		
		
	player = new XA_Player();
		if(player == NULL) fFlag = true;
	player->Run(NULL);
	pControl = player->control;
	rxfd = static_cast<XA_PipeControl*>(pControl)->command_read_pipe;

// sets up the file descriptor set for the select function in the main loop.
	FD_ZERO(&set);
  FD_SET(sfd, &set);
	FD_SET(rxfd, &set);
// sets up the display for specific functionality, noted on the right
	fprintf(fPort,"%cC",commandPrefix);        // Setup autoline wrapping
	fprintf(fPort,"%cQ",commandPrefix);        // Sets up auto scrolling
	fprintf(fPort,"%c~%c",commandPrefix,0x00); // Sets up  Auto Repeat Mode
	ClearScreen();
	SetSong("Welcome to SunAmp - Press >or< to Start");
	player->SetPlayerMode(XA_PLAYER_MODE_OUTPUT_AUTO_CLOSE_ON_STOP |
		XA_PLAYER_MODE_OUTPUT_AUTO_CLOSE_ON_PAUSE);
	player->SetCodecEqualizer(&eqInfo);
	PlayScreen(0);
};

Display::~Display()
{
	if(serPort)
	{
		serPort->SetupDefault();
		serPort->ClearDTR();
		serPort->Close();
		delete serPort;
		serPort = NULL;
	}
	if(player)
	{
		player->Stop();
		player->InputClose();
		player->Exit();
		delete player;
	 	player = NULL;
	}
	if(songList)
	{
		delete songList;	
		songList = NULL;
	}
	if(songDir)
	{
		delete songDir;
		songDir = NULL;
	}
};

int Display::SetSongDir(char *sd)
{
	songDir = new char[strlen(sd)+1];
	if(songDir == NULL)
	{
		return -1;
	}
	strcpy(songDir,sd);
	songList = new SongList(songDir);
	if(songList->Fail())
	{
		cout << "Error Creating SongList" << endl;
		return -1;
	}
	memset(currentSong,0,sizeof(currentSong));
	songList->SetRandom(true);
	songList->GetNextSong(currentSong,100);
	PlayScreen(0);
	return 0;
}

int Display::GetDescriptorSet(fd_set * fds1)
{
	*fds1 =  set;
	return 0;

}

int Display::GetFileDescriptors(int &fd1, int & fd2)
{
	if(rxfd != 0)fd1 = rxfd;
	if(sfd != 0)fd2 = sfd;
	return 0;
}

int Display::Event(EVENT event)
{
	int input;
	switch(event)
	{
		case XAUDIO:
			if(control_message_get(pControl,&message) > 0)
			{
				if(ProcessXaudioMessage(message))
					return -1;
			}	
			break;
		case SERIAL:
			input = GetKey();
			if (input > 0)
			{
				if(ProcessKeyPress(input))
					return -1;
			}
			break;	
		case REMOTE:
			input = remotePointer->GetKey();
			if(input > 0)
			{
				if(ProcessKeyPress(input))		
					return -1;
			}
			break;
		case TIMEOUT:
			switch(dispMode)
			{
				case PLAY: PlayScreen(2);break;
				case MENU: MenuScreen(2);break;
				case EQ:   EqualizerScreen(2);break;
				case LISTS: ListsScreen(2);break;
				case CONTRAST: ContrastScreen(2);break;
				default: break;
			}
		default: break;
	}
	return 0;
}


int Display::ProcessKeyPress(int input)
{
	int modeSwitch = false;
	if(input == 'B')
	{
		modeSwitch = true;
		switch(dispMode)
		{
			case PLAY:	   dispMode = MENU;MenuScreen(0);break;
			case MENU:     dispMode = PLAY;PlayScreen(0);break;
			case EQ:       dispMode = MENU;MenuScreen(0);break;
			case LISTS:    dispMode = MENU;MenuScreen(0);break;
			case CONTRAST: dispMode = MENU;MenuScreen(0);break;
		}
		
	}
	else
	{
 		switch(dispMode)
		{
			case PLAY: PlayScreen(input);break;
			case MENU: MenuScreen(input);break;
			case EQ:   EqualizerScreen(input);break;
			case LISTS: ListsScreen(input);break;
			case CONTRAST: ContrastScreen(input);break;
			default:
				break;
		}
	}
	return 0;
}

int Display::PlayScreen(int input)
{
	char temp[100];
	int stringLen;
	bool autoPlay;
	switch(input)
	{
		case 0: //setup screen
				ClearScreen();
				fprintf(fPort,"%cH%s",commandPrefix,artistLine.scrolled);
				fprintf(fPort,"%cG%c%c",commandPrefix,1,2);
				fprintf(fPort,"%s",songLine.scrolled);
				break;
		case 1: // display info
				if(dispMode == PLAY)
				{
					fprintf(fPort,"%cG%c%c",commandPrefix,6,3);
					fprintf(fPort,"%02d:%02d(%02d%%)",streamMinutes,streamSeconds,streamProgress);
					fprintf(fPort,"%cG%c%c",commandPrefix,5,4);
					fprintf(fPort,"%03dKB/s %dKHz",streamBitrate,streamFrequency);	
				}
				break;		
		case 2:
			memset(temp,0,sizeof(temp));			
			if(songLine.scrolling)
			{		
				if(songLine.currentPosition >= songLine.length)
					songLine.currentPosition = 0;
				strncpy(temp,&songLine.string[songLine.currentPosition], displayLength);
				stringLen = strlen(temp);
        if(stringLen < displayLength)
        	strncpy(&temp[stringLen],songLine.string,(displayLength - stringLen));
				strncpy(songLine.scrolled,temp,displayLength);
        songLine.scrolled[displayLength] = (char)NULL;
				songLine.currentPosition++;
			}
			if(artistLine.scrolling)
			{		
				if(artistLine.currentPosition >= artistLine.length)
					artistLine.currentPosition = 0;
				strncpy(temp,&artistLine.string[artistLine.currentPosition], displayLength);
				stringLen = strlen(temp);
        if(stringLen < displayLength)
        	strncpy(&temp[stringLen],artistLine.string,(displayLength - stringLen));
				strncpy(artistLine.scrolled,temp,displayLength);
        artistLine.scrolled[displayLength] = (char)NULL;
				artistLine.currentPosition++;
			}
			fprintf(fPort,"%cH%s",commandPrefix,artistLine.scrolled);
			fprintf(fPort,"%cG%c%c",commandPrefix,1,2);
			fprintf(fPort,"%s",songLine.scrolled);
			break;
		case 'A': // play/pause
				if(playerState == XA_PLAYER_STATE_PLAYING)
					player->Pause();
				else if(playerState == XA_PLAYER_STATE_PAUSED)
				{
						player->Play();
						player->SetCodecEqualizer(&eqInfo);
				}
				//if((streamInputState == XA_IO_STATE_OPEN) &&
				//	(playerState !=
				else if(playerState != XA_PLAYER_STATE_PLAYING)
				{					
				  SaySong();
					player->Play();
					player->SetCodecEqualizer(&eqInfo);
				}
				break;
		case 'C': // stop
				if(playerState == XA_PLAYER_STATE_PLAYING)
				{
					player->Stop();
					
					//player->InputClose();
				}
				break;
		case 'D': // next
				autoPlay = false;
				if(playerState == XA_PLAYER_STATE_PLAYING ||
						playerState == XA_PLAYER_STATE_PAUSED ||
						playerState == XA_PLAYER_STATE_EOF)
				{
					player->Stop();
					player->InputClose();
					autoPlay = true;
				}
				if(streamInputState == XA_IO_STATE_OPEN)
					player->InputClose();
				songList->GetNextSong(currentSong,100);
				strcpy(playerSong,songDir);
				strcat(playerSong,currentSong);
				player->InputOpen(playerSong);
				if(autoPlay)
				{
				  SaySong();
					player->Play();
					player->SetCodecEqualizer(&eqInfo);
				}
				SetSong(currentSong);
				if(dispMode == PLAY)
				{
					ClearScreen();
					fprintf(fPort,"%cH%s",commandPrefix,artistLine.scrolled);
					fprintf(fPort,"%cG%c%c",commandPrefix,1,2);
					fprintf(fPort,"%s",songLine.scrolled);
				}
				break;
		case 'E': // previous
				autoPlay = false;
				if(playerState == XA_PLAYER_STATE_PLAYING ||
						playerState == XA_PLAYER_STATE_PAUSED ||
						playerState == XA_PLAYER_STATE_EOF)
				{
					player->Stop();
					player->InputClose();
					autoPlay = true;
				}
				if(streamInputState == XA_IO_STATE_OPEN)
					player->InputClose();
				songList->GetPrevSong(currentSong,100);								
				strcpy(playerSong,songDir);
				strcat(playerSong,currentSong);
				player->InputOpen(playerSong);
				if(autoPlay)
				{
					SaySong();
					player->Play();
					player->SetCodecEqualizer(&eqInfo);	
				}
				SetSong(currentSong);
				if(dispMode == PLAY)
				{
					ClearScreen();
					fprintf(fPort,"%cH%s",commandPrefix,artistLine.scrolled);
					fprintf(fPort,"%cG%c%c",commandPrefix,1,2);
					fprintf(fPort,"%s",songLine.scrolled);
				}
				break;
	}
	fflush(fPort);
	return 0;
}
int Display::MenuScreen(int input)
{
	static int cursorPosition;
	switch(input)
	{
		case 0: // just entered, draw list
			ClearScreen();
			if(cursorPosition < 1 || cursorPosition > 4)
				cursorPosition = 1;
			fprintf(fPort,"  Equalizer\n");
			fprintf(fPort,"  PlayList\n");
			fprintf(fPort,"  Contrast\n");
			fprintf(fPort,"  Shutdown");
			fprintf(fPort,"%cG%c%c*",commandPrefix,1,cursorPosition);
			break;
		case 'D': // moves cursor up
			fprintf(fPort,"%cG%c%c ",commandPrefix,1,cursorPosition);
			cursorPosition--;
			if(cursorPosition < 1) cursorPosition = 4;
			fprintf(fPort,"%cG%c%c*",commandPrefix,1,cursorPosition);
			break;
		case 'E': // moves cursor down
			fprintf(fPort,"%cG%c%c ",commandPrefix,1,cursorPosition);
			cursorPosition++;
			if(cursorPosition > 4) cursorPosition = 1;
			fprintf(fPort,"%cG%c%c*",commandPrefix,1,cursorPosition);
			break;
		case 'A': // executes function
			switch(cursorPosition)
			{
				case 1: dispMode = EQ;
					EqualizerScreen(0);
					break;
				case 2: dispMode = LISTS;
					ListsScreen(0);
					break;
				case 3: dispMode = CONTRAST;
					ContrastScreen(0);
					break;
				case 4: quit = true;
					break;
				default: break;
			}
	}
	fflush(fPort);
	return 0;
}

int Display::ListsScreen(int input)
{
	switch(input)
	{
		case 0: //scroll song name if applicable
				ClearScreen();
				fprintf(fPort,"Lists Screen");
				break;
		case 'A': // play/pause
				break;
		case 'C': // stop
				break;
		case 'D': // next
				break;
		case 'E': // previous
				break;
	}
	fflush(fPort);
	return 0;
}
int Display::ContrastScreen(int input)
{
	int hundreds, tens, ones;
	switch(input)
	{
		case 0:
				ClearScreen();
				hundreds = contrast /100;
				tens = (contrast - (hundreds*100))/10;
				ones = (contrast - ((tens*10) + (hundreds*100)));
				fprintf(fPort,"Set\nContrast");
				fprintf(fPort,"%cn",commandPrefix);
				fprintf(fPort,"%c#%c%c",commandPrefix,10,hundreds);
				fprintf(fPort,"%c#%c%c",commandPrefix,14,tens);
				fprintf(fPort,"%c#%c%c",commandPrefix,18,ones);
				break;
		case 'D': // contrast up
				SetContrast(UP);
				hundreds = contrast /100;
				tens = (contrast - (hundreds*100))/10;
				ones = (contrast - ((tens*10) + (hundreds*100)));
				fprintf(fPort,"%cn",commandPrefix);
				fprintf(fPort,"%c#%c%c",commandPrefix,10,hundreds);
			  fprintf(fPort,"%c#%c%c",commandPrefix,14,tens);
				fprintf(fPort,"%c#%c%c",commandPrefix,18,ones);
      	break;
		case 'E': // contrast down
				SetContrast(DOWN);
				hundreds = contrast /100;
				tens = (contrast - (hundreds*100))/10;
				ones = (contrast - ((tens*10) + (hundreds*100)));
				fprintf(fPort,"%cn",commandPrefix);
				fprintf(fPort,"%c#%c%c",commandPrefix,10,hundreds);
			  fprintf(fPort,"%c#%c%c",commandPrefix,14,tens);
				fprintf(fPort,"%c#%c%c",commandPrefix,18,ones);
				break;
		default:
		break;
	}	
	fflush(fPort);
	return 0;
}


int Display::EqualizerScreen(int input)
{
	int temp;
	static int currentBand;
	switch(input)
	{
		case 'A': // eq band select
				currentBand++;
				if(currentBand > 9)
					currentBand = 0;	
				fprintf(fPort,"%cG%c%cBand:%02d",commandPrefix,14,3,currentBand+1);			
				fprintf(fPort,"%cG%c%cFreq:%s",
					commandPrefix,11,4,freq[currentBand]);
				break;
		case 'C': // eq band select
				currentBand--;
				if(currentBand < 0)
					currentBand = 9;	
				fprintf(fPort,"%cG%c%cBand:%02d",commandPrefix,14,3,currentBand+1);			
				fprintf(fPort,"%cG%c%cFreq:%s",
					commandPrefix,11,4,freq[currentBand]);
				break;

		case 'D': // eq band upup
				eqbands.eqbands[currentBand]+=8;
				if(eqbands.eqbands[currentBand] > 127)
					eqbands.eqbands[currentBand] = 127;
				if(eqbands.eqbands[currentBand] <= 0)
				{
					temp = 16 - (abs(eqbands.eqbands[currentBand])/8);
				}
				else
				{
					temp = 16 + (eqbands.eqbands[currentBand]/8) + 1;
				}

				for(int i=0;i<6;i++)
				{
					eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[i];
				}
				eqInfo.left[6]= eqInfo.right[6] = eqInfo.left[7] =
				eqInfo.right[7] = eqbands.eqbands[6];
				for(int i=8;i<=11;i++)
				{
					eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[7];
				}		
				for(int i=12;i<=19;i++)
				{
					eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[8];
				}		
				for(int i=20;i<=31;i++)
				{
					eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[9];
				}		
				player->SetCodecEqualizer(&eqInfo);

				fprintf(fPort,"%cv",commandPrefix);
				fprintf(fPort,"%c=%c%c",commandPrefix,currentBand+1,temp);
				break;
		case 'E': // eq band down
				eqbands.eqbands[currentBand]-=8;
				if(eqbands.eqbands[currentBand] < -128)
					eqbands.eqbands[currentBand] = -128;

				if(eqbands.eqbands[currentBand] <= 0)
				{
					temp = 16 - (abs(eqbands.eqbands[currentBand])/8);
				}
				else
				{
					temp = 16 + (eqbands.eqbands[currentBand]/8) + 1;
				}


				for(int i=0;i<6;i++)
				{
					eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[i];
				}
				eqInfo.left[6]= eqInfo.right[6] = eqInfo.left[7] =
				eqInfo.right[7] = eqbands.eqbands[6];
				for(int i=8;i<=11;i++)
				{
					eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[7];
				}		
				for(int i=12;i<=19;i++)
				{
					eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[8];
				}		
				for(int i=20;i<=31;i++)
				{
					eqInfo.left[i] = eqInfo.right[i] = eqbands.eqbands[9];
				}		
				player->SetCodecEqualizer(&eqInfo);
 				fprintf(fPort,"%cv",commandPrefix);
				fprintf(fPort,"%c=%c%c",commandPrefix,currentBand+1,temp);
				break;
		case 0: // just entered;
				ClearScreen();
				fprintf(fPort,"%cG%c%cGraphic",commandPrefix,14,1);
				fprintf(fPort,"%cG%c%cEq",commandPrefix,16,2);
				fprintf(fPort,"%cG%c%cBand:%02d",commandPrefix,14,3,currentBand+1);
				fprintf(fPort,"%cG%c%cFreq:%s",
					commandPrefix,11,4,freq[currentBand]);
				fprintf(fPort,"%cv",commandPrefix);
				for(int i=1;i<=10;i++)
				{
					if(eqbands.eqbands[i-1] <= 0)
					{
						temp = 16 - (abs(eqbands.eqbands[i-1])/8);
					}
					else
					{
						temp = 16 + (eqbands.eqbands[i-1]/8) + 1;
					}
					fprintf(fPort,"%c=%c%c",commandPrefix,i,temp);
				}
				break;
	}
	fflush(fPort);
	return 0;
}
	

int Display::ProcessXaudioMessage(const XA_Message & message)
{
	switch(message.code)
	{
		case XA_MSG_NOTIFY_READY:
				playerReady = true;
				break;
		case XA_MSG_NOTIFY_EXITED:
				playerReady = false;
				break;
		case XA_MSG_NOTIFY_PLAYER_STATE:
				playerState = message.data.state;
				if(playerState == XA_PLAYER_STATE_EOF)
					PlayScreen('D');
				break;
		case XA_MSG_NOTIFY_INPUT_POSITION:
				streamProgress = int((double(message.data.position.offset)/
					double(message.data.position.range)) * 100.0);
					PlayScreen(1);
				break;
		case XA_MSG_NOTIFY_INPUT_TIMECODE:
				streamMinutes = message.data.timecode.m;
				streamSeconds = message.data.timecode.s;	
				PlayScreen(1);
				break;
		case XA_MSG_NOTIFY_INPUT_STREAM_INFO:
				streamBitrate = message.data.stream_info.bitrate;
				streamFrequency = message.data.stream_info.frequency/1000;
				streamMode = message.data.stream_info.mode;
				PlayScreen(1);
				break;
		case XA_MSG_NOTIFY_INPUT_STATE:
				streamInputState = message.data.state;
				break;
		case XA_MSG_NOTIFY_ERROR:
		case XA_MSG_NOTIFY_NACK:
					//char string[100];
					//control_message_sprint(string,&message);
					cout << "Error: " << int(message.code) << endl;
					return -1;
		default: break;
	}
	newXAudioData = true;
	return 0;
}

int Display::GetKey()
{
	return getc(fPort);
}

void Display::ClearScreen()
{
	fprintf(fPort,"%cX",commandPrefix);
	fflush(fPort);
}

void Display::BackLightOff(int minutes)
{
	if((minutes > 0) && (minutes <= 180))
	{
		fprintf(fPort,"%cB%c", commandPrefix, minutes);
		fflush(fPort);
	}
}

void Display::SetContrast(CONT dir)
{
	
	if(dir == UP && ((contrast + contStep) < 256))
	{
		contrast += contStep;
		fprintf(fPort,"%cP%c",commandPrefix,contrast);
	}
	
	if(dir == DOWN && ((contrast - contStep) > 0))
	{
		contrast -= contStep;
		fprintf(fPort,"%cP%c",commandPrefix,contrast);
	}
	fflush(fPort);
}

void Display::SetSong(char * s)
{
	char * temp3;
  int i,j,k;
  int length = strlen(s);

	songLine.currentPosition = 0;
	artistLine.currentPosition = 0;
	if((length <= 0) || (length >= 100))
	{
		songLine.string[0]=0;
		artistLine.string[0]=0;
		artistLine.scrolled[0]=0;
		songLine.string[0]=0;
    return;
	}

  for(i = 0; (s[i] != '-') && (i < length); i++);

  if(i >= length)
  {
		strcpy(songLine.string,s);
		if(length > displayLength)
		{
			songLine.scrolling = true;
			artistLine.scrolling = false;
			memset(songLine.scrolled,0,sizeof(songLine.scrolled));
			memset(artistLine.scrolled,0,sizeof(artistLine.scrolled));
			strncpy(songLine.scrolled,songLine.string,displayLength);
			strcat(songLine.string," * ");
			songLine.length = strlen(songLine.string);
		}
		else
		{
			songLine.scrolling = false;  	
			artistLine.scrolling = false;
			memset(songLine.scrolled,' ',sizeof(songLine.scrolled));
			memset(artistLine.scrolled,' ',sizeof(artistLine.scrolled));
      songLine.scrolled[20] = (char)NULL;
      k = 10 - (strlen(songLine.string)/2);
      if( k > 0)
 	     strncpy(&songLine.scrolled[k],songLine.string,displayLength);
     songLine.scrolled[displayLength] = (char)NULL;
		 songLine.length = strlen(songLine.string);			
		}	
	}
  else
	{
		memset(artistLine.string,0,sizeof(artistLine.string));
		memset(songLine.string,0,sizeof(songLine.string));
		j = i;
    strncpy(artistLine.string,s,i-1);
		j = length - i;
		strncpy(songLine.string,&s[i+2],j);
		//temp3 = strstr(songLine.string,".mp3");
		temp3 = strrchr(songLine.string,'.');
    if(temp3)
    {
    	*temp3 = char(NULL);
    }

		if(strlen(songLine.string) > (unsigned)displayLength)
		{
			songLine.scrolling = true;
			memset(songLine.scrolled,0,sizeof(songLine.scrolled));
			strncpy(songLine.scrolled,songLine.string,displayLength);
			strcat(songLine.string," * ");
			songLine.length = strlen(songLine.string);			
		}
		else
		{
			songLine.scrolling = false;
			memset(songLine.scrolled,' ',sizeof(songLine.scrolled));
      songLine.scrolled[20] = (char)NULL;
      k = 10 - (strlen(songLine.string)/2);
      if( k >= 0)
 	     strncpy(&songLine.scrolled[k],songLine.string,displayLength);
     songLine.scrolled[displayLength] = (char)NULL;
		 songLine.length = strlen(songLine.string);
			
		}	

		if(strlen(artistLine.string) > (unsigned)displayLength)
		{
			artistLine.scrolling = true;
			memset(artistLine.scrolled,0,sizeof(artistLine.scrolled));
			strncpy(artistLine.scrolled,artistLine.string,displayLength);
			strcat(artistLine.string," * ");
			artistLine.length = strlen(artistLine.string);
		}
		else
		{
			artistLine.scrolling = false;
			memset(artistLine.scrolled,' ',sizeof(artistLine.scrolled));
      artistLine.scrolled[20] = (char)NULL;
      k = 10 - (strlen(artistLine.string)/2);
      if( k >= 0)
 	     strncpy(&artistLine.scrolled[k],artistLine.string,displayLength);
     artistLine.scrolled[displayLength] = (char)NULL;			
		 artistLine.length = strlen(artistLine.string);
		}		
	}	
	return;
};

int Display::SaySong()
{
	char temp[100];
	char * period;

	strcpy(temp,currentSong);
	period = strrchr(temp,'.');
  if(period != NULL)
	{
		*period = char(NULL);
  }
	period = strrchr(temp,'-');
	if(period != NULL)
	{
		*period = ',';
	}
	return festival_say_text(temp);
}

void Display::SetRemotePointer(Remote * rp)
{
	remotePointer = rp;
}
