/***************************************************************************
                          songlist.cpp  -  description
                             -------------------
    begin                : Sat Jul 1 2000
    copyright            : (C) 2000 by Calvin Harrigan
    email                : charrig@viaccess.net
		See the header files for details
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <algorithm>
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include <unistd.h>
#include <time.h>
#include <dirent.h>
#include <sys/stat.h>
#include "songlist.h"


// This function is used by the quicksort function to sort
// the objects in the array.  In this case Song objects.
int mycmp(const void * ptr1, const void * ptr2)
{
	Song ** p1;
	Song ** p2;
	
	p1 = (Song**)ptr1;
	p2 = (Song**)ptr2;
	return strcmp( (*p1)->name,(*p2)->name);
};

SongList::SongList(char * directory)
{
	//class variables
	fFlag = false;
	songs = NULL;
	songCount = 0;
	songIndex = 0;
	random = true;
	errorMessage.name = "No Songs Selected - Error Message";
	errorMessage.path = "No Songs Selected - Error Message.mp3";

	//local variables
	DIR * dp;              // directory pointer
	struct dirent *ep;     // directory entry pointer

	struct stat statbuff;
	int n = 0;             // count of directories
	int i = 0;             // General Purpose index

	randomize();           // start psuedo random generator

	if(chdir(directory))   // change the working directory of the app.
	{                      // check for a successful change, returns on fail
		fFlag = true;
		cout << "Cannot change working directory." << endl;
		return;
	}
	dp = opendir(directory);    // open the given directory returns of fail

	if(dp != NULL)              // reads directory entries to get a count
	{
		while((ep = readdir(dp)))
		{
			if(stat(ep->d_name,&statbuff))  // flags failed stat calls.
				cout << "Couldn't stat: " << ep->d_name << endl;
			else
			{  // makes sure the entry is a directory other than . and ..
			  if(S_ISDIR(statbuff.st_mode) && (ep->d_name[0] != '.'))
					n++;                 //n counts the amount of subdirectories
			}
		}		
	}
	else
	{
		fFlag = true;              // if opening the directory fails
   	return;
	}                            // return

  rewinddir(dp);               // rewind to start of directory entries
	dlist = new Directory * [n]; // create an array of Directory pointers to
	if(dlist)                    // check for successful allocation
	{
		while((ep = readdir(dp)))
		{
			if(stat(ep->d_name,&statbuff))
				cout << "Couldn't 2nd stat: " << ep->d_name << endl;
			else
			{
			  if(S_ISDIR(statbuff.st_mode))
				{
					if(strcmp(".",ep->d_name) && strcmp("..",ep->d_name))
					{ // creates a new Dlist object for each directory except . and ..
						dlist[i] = new Directory(ep->d_name);
						if( dlist[i]->Fail() )
						{
							fFlag = true;    //flags failure and returns
							return;
						}
						i++;              // i indexes objects
					}					
				}
			}
		}
	}		
  genreCount = i;                // stores the count of directories
	if(Update())
	{
		fFlag = true;
		return;
	}
};

SongList::~SongList()
{
	// deletes the allocated objects
	// first the directory objects then the array
	// of directory pointers.
	for(int i = 0; i < genreCount; i++)
	{
		delete dlist[i];
	}
	delete dlist;
	dlist = NULL;
	delete songs;
	songs = NULL;
};

int SongList::Update()
{
	int totalSongCount = 0;
  // If songs already exists, delete it, set it to null.
	if(songs)
	{
		delete songs;
		songs = NULL;
		totalSongCount = 0;
	}

	// Count the number of available songs

	for(int i = 0; i < genreCount; i++)
	{
		if(dlist[i]->GetAvailable())
			totalSongCount += dlist[i]->GetSongCount();
	}
	// Allocate that much memory...
	songs = new Song * [totalSongCount];
	// fail is memory allocations is unsuccessful.
	if(!songs)
	{
		return -1;
	}

	int n = 0;      // indexes through the array
	Song ** dSongs; // temporary pointer to the songs in the directory
	int dCount;     // temporarily holds the song count in the directory
	for(int i = 0; i < genreCount; i++)  //iterrate through the directory
	{                                    // objects checking for availability
		if(dlist[i]->GetAvailable())       // retrieve the pointer to the array
		{                                  // and the amount of elements in it
			dSongs = dlist[i]->GetSongs();   // then simply copy the pointers to the
			dCount = dlist[i]->GetSongCount();// new list.  The variable is incremented			
			for(int j = 0; j < dCount; j++)   // for everysong in every directory.
			{
				songs[n] = dSongs[j];           // actual pointer copy.
				n++;
			}
		}
	}		
  songCount = totalSongCount;           // save the song count in a class
                                        // variable.
	if(!random)                                 // if the not set
	{                                      // sort the songs...
		qsort(songs,songCount,sizeof(Song*),mycmp);
	}
	else                                   // else shuffle them.
	{
		Shuffle(); //calls the private shuffle function
	}
	return 0;
};

int SongList::GetNextSong(Song* & s)
{                                     // should be self explanatory
	if(songCount == 0)
	{
		s = &errorMessage;
		return 0;
	}
	songIndex++;
	if(songIndex >= songCount)
		songIndex = 0;
	s = songs[songIndex];
	return 0;
};

int SongList::GetPrevSong(Song* & s)
{                                    // should be explanatory
	if(songCount == 0)
	{
		s = &errorMessage;
		return 0;
	}
	songIndex--;
	if(songIndex < 0)
		songIndex = songCount-1;
	s = songs[songIndex];
	return 0;
};

int SongList::SetAvailable(bool a, int genre)
{                                    // sets directory/genre available
	if((genre > 0) && genre <= (genreCount))       // or not.  Checks
	{                                              // for valid entries.
		dlist[genre]->SetAvailable(a);
		return 0;
	}
	return -1;	
};

int SongList::Shuffle()
{
	int i;
	int randValue;
	Song * temp;
	// Checks for valid values for the following
	// returns -1 on failure
	if((songs == NULL) || (songCount <= 0))
		return -1;
	// seeds the random number generator with the current time
	randomize();
	for(i=0;i<songCount;i++)
	{
		// swaps the currently indexed value with a random index that is greater
		// very cheap shuffle algo found somewhere on the net and modified to suit.
		randValue = (rand()%(songCount-i)+i);
		temp = songs[i];
		songs[i] = songs[randValue];
		songs[randValue] = temp;		
	}
	return 0;
};
