/***************************************************************************
                          songlist.h  -  description
                             -------------------
    begin                : Sat Jul 1 2000
    copyright            : (C) 2000 by Calvin Harrigan
    email                : charrig@viaccess.net

The songlist class is used to index the songs in the directory passed as an
argument from the display class.  It searches for directories first and
assumes that they contain songs sorted by genre.  It then compiles seperate
directory classes for each.  The directory classes are then used to create a
pool of songs that can be accessed through the songlist class interface using
some pointer magic.  The songlist class allows the songs to be sorted or randomly
arranged.
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __SONGLIST_H__
#define __SONGLIST_H__

// Some macros used to manipulate the srand and rand functions

#ifndef random
	#define random(num)(int) ((long)rand()%(num))
#endif

#ifndef randomize
	#define randomize() srand((unsigned)time(NULL))
#endif


// Includes the directory class that is used throughout
#include "directory.h"


class SongList
{
	private:
		int Shuffle();      // Shuffles the songpointers in the songs array.
		int genreCount;     // Count of the amount of genres/directory in the home directory.
		Directory ** dlist; // A pointer to array of Directory objects
		Song ** songs;      // A pointer to array of Song Pointers.
		bool fFlag;         // Fail Flag
		int songCount;      // The amount of songs in the "songs" array.
		int songIndex;      // Currently selected song in the "songs" array.
		bool random;        // Random/Sorted flag
		Song errorMessage;  // Message that is played when an empty songlist error occurs
	public:
		SongList(){};               // Default constructor
		SongList(char * directory); // Main constructory
		~SongList();                // Destructor
		int GetNextSong(Song * &);  // Returns the next song in the list
		int GetPrevSong(Song * &);  // Returns the previous song in the list
		int SetAvailable(bool a, int genre);  // Sets a particular genre available in the playlist
		Directory ** GetGenreList()const{return dlist;}; // returns the array of directories
		int GetGenreCount()const{return genreCount;};  // returns the count of directory objects in the array above.
		void SetRandom(bool r){random = r;Update();}; // changes the random flag
		int Update();                                 // resorts the songs in the songlist
		int GetSongCount()const{return songCount;};   // return the count of songs in the "songs" array
		bool Fail()const{return fFlag;};              // returns the fail flag
		int GetRandom()const{return random;};         // returns random status
};

#endif // __SONGLIST_H__
//============END OF FILE=====================		