/***************************************************************************
                          sunamp.cpp  -  description
                             -------------------
    begin                : Mon Jun 5 2000
    copyright            : (C) 2000 by Calvin Harrigan
    email                :  charrig@viaccess.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <sys/time.h>
#include "display.h"
#include "remote.h"

// Helper function that does some timeval structure math

int timeSubtract(struct timeval const & t1, struct timeval const & t2);


int main()
{
	Display *  disp;        // Display class pointer
	Remote * remote;        // Remote class pointer
	fd_set fdset, set;      // Descriptor sets used in select
	int rxfd, sfd, rfd;     // file descriptors for pipe and comm ports
	int sResult;            // Select return value
	int selectTime = DRATE; // Select timeout value (varies with input)

// create a new instance of the display class and check for validity.
// I know the comm port shouldn't be hard coded, but times hard...  :-)

	disp = new Display("/dev/ttyS1");
	if(disp->Fail())
	{
		delete disp;
		cout << "Failed to create display object" << endl;
		return -1;
	}
	if(disp->SetSongDir("/mp3s"))
	{
		delete disp;
		cout << "Error setting up song directory" << endl;
		return -1;
	}
	remote = new Remote("/dev/ttyS0");
	if(remote->Fail())
	{
		delete remote;
		//delete disp;  // No need to kill the entire process
		//return -1;    // if the remote doesn't work.
		disp->SetRemotePointer(NULL);
		rfd = (int)NULL;
	}
	else
	{
		disp->SetRemotePointer(remote);
		rfd = remote->GetFileDescriptor();
	}

// get the file descriptors for xaudio pipe and the serial interface
	disp->GetFileDescriptors(rxfd,sfd);
//******************************************************************************
// Set the descriptor sets to zero and initialize one with the new descriptors	
	FD_ZERO (&set);
	FD_ZERO (&fdset);
	FD_SET(rxfd, &fdset);
	FD_SET(sfd, &fdset);
  FD_SET(rfd, &fdset);
// time structures used for timing out select and calculating elapsed time	
	struct timeval timeout,displayTime1,displayTime2;

// Initialize the timeout data structure.
        timeout.tv_sec = 0;
        timeout.tv_usec = DRATE;
        set = fdset;

// get the time of day to calculate elapsed time
	gettimeofday(&displayTime1,NULL);
	gettimeofday(&displayTime2,NULL);
	
// select returns 0 if timeout, 1 if input available, -1 if error.
// Continue the loop while the input doesn't request a quit.
	while(!disp->Quit())
	{
// the fd set and timout structures have to be reinitialized
// after ever select.  The fd_set contains info about the
// available input.   		
		set = fdset;
		timeout.tv_sec = 0;
		timeout.tv_usec = selectTime;
// get current time to calculate elapsed time for an even
// time spread, wheter timed out of interrupted.
		gettimeofday(&displayTime1,NULL);
		sResult = select (FD_SETSIZE, &set, NULL, NULL,&timeout);
// check return value of select and dispatch the right event
// if Select returns a value greater than 0, one or more file
// descriptors are ready for reading.
		if(sResult > 0)
		{
			if(FD_ISSET(rxfd,&set))
			{
				if(disp->Event(disp->XAUDIO))
					cout << "Error on dispatching XAudio Event." << endl;
			}
			if(FD_ISSET(sfd,&set))
			{
				if(disp->Event(disp->SERIAL))
					cout << "Error on dispatching Serial Event." << endl;
			}
			if(FD_ISSET(rfd, &set))
			{
				if(remote->Event())
					cout << "Error on dispatching Remote Event 1." << endl;
				else
					if(disp->Event(disp->REMOTE))
						cout << "Error on dispatching Remote Event 2." << endl;
			}
			// Calculate the time remaining in the interval;
			gettimeofday(&displayTime2,NULL);
			if(timeSubtract(displayTime2,displayTime1))
				selectTime = selectTime - (timeSubtract
					(displayTime2,displayTime1));
			// Prevent negative Select values
			if(selectTime < 0)selectTime = 0;
		}
		// if 0 is returned, Select just time out;
		if(sResult == 0)
		{
			if(disp->Event(disp->TIMEOUT))
				cout << "Error on dispatching TimeOut Event." << endl;	
			selectTime = DRATE;
		}
		// if a negative value is returned an error occured
		if(sResult < 0)
			cout << "File Descriptor Select Error." << endl;
	}				
	
//*************************************************************************
// Clears screen gives the class some time to clean up, delete the
// class then quit	
        disp->ClearScreen();
	sleep(1);   	
	delete disp;
	return 0;
};

// Self explanatory.

int timeSubtract(struct timeval const & t1, struct timeval const & t2)
{
	int temp1 = t1.tv_sec * 1000000 + t1.tv_usec;
	int temp2 = t2.tv_sec * 1000000 + t2.tv_usec;
	if(temp1 > temp2)
		return (temp1 - temp2);
	else
		return (temp2 - temp1);
	return 0;
}
